/*
 *	LibGame - Copyright (C) Emmanuel Thomas-Maurin 2011-2025
 *	All rights reserved
 */

#ifndef LG_LANDSCAPE_H
#define LG_LANDSCAPE_H

#define QUADTREE_NAME_MAX_LEN		(64 - 1)

#define QUADTREE_MAX_DEPTH		32
#define QUADTREE_PRINTOUT_MAX_DEPTH	8

#define QUADTREE_TO_SCENENODE_ID_SHIFT	1000

/* QuadTree quadrants */
typedef enum {
	LG_NW = 0,
	LG_NE,
	LG_SW,
	LG_SE,
	LG_QUADRANT_UNDEFINDED
} lg_quadrant;

/* LG_QTNode = LG QuadTree Node */
typedef struct LG_QTNode LG_QTNode;

struct LG_QTNode {
	int		id;					/* User-defined */
	char		name[QUADTREE_NAME_MAX_LEN + 1];
	LG_QTNode	*parent;
	LG_QTNode	*child[4];				/* N-W, N-E, S-W, S-E */
	int		quadrant;				/* LG_NW, LG_NE, LG_SW, LG_SE */
	Rec2Df		bounds;					/* Actually a bounding square, ie w = h */
	Point2Df	center;
	float		altitude;
	zboolean	is_leaf;
	void		*data;					/* Usually a mesh */
};

LG_QTNode	lg_qtnode(int, const char *, Rec2Df *,  void *);

void		lg_qtnode_set_altitude(LG_QTNode *, float);

void		lg_qtnode_add_child(LG_QTNode *, LG_QTNode *, int, float);

LG_QTNode	*lg_qtnode_get_root(LG_QTNode *);

void		lg_add_scenenode_from_qtnode(LG_SceneNode *, LG_QTNode *);

void		lg_traverse_landscape(LG_QTNode *, int (*)(LG_QTNode *));

void		lg_traverse_printout_landscape(LG_QTNode *, zboolean);

void		lg_qtnode_info(LG_QTNode *);

int		lg_qtnode_info2(LG_QTNode *);

#endif /* LG_LANDSCAPE_H */
