/*
 *	LibGame - Copyright (C) Emmanuel Thomas-Maurin 2011-2025
 *	All rights reserved
 */

#ifndef LG_LIGHT_H
#define LG_LIGHT_H

typedef enum {
	POINT,
	DIRECTIONAL,
	SPOTLIGHT,
	AMBIENT,
	DIFFUSE,
	EMISSIVE,
	SPECULAR
} lg_light_type;

#define WHITE_LIGHT	(LG_Color_f){1.0, 1.0, 1.0, 1.0}

/*
 * LG_Light struct
 *
 * Use an array of LG_Light structs if you want to have several different lights in a scene
 */
typedef struct {
	lg_light_type	type;
	union {			// === NOT SURE THIS IS OK ===
		vec3_t	pos;
		vec3_t	dir;
	};
	LG_Color_f	color;
} LG_Light;

LG_Light	lg_light(lg_light_type, vec3_t, LG_Color_f);

void		lg_set_light_src(LG_Light *, vec3_t);

vec3_t		lg_get_light_src(LG_Light *);

void		lg_move_light_src(LG_Light *, vec3_t);

void		lg_set_light_dir(LG_Light *, vec3_t);

vec3_t		lg_get_light_dir(LG_Light *);

void		lg_change_light_dir(LG_Light *, LG_EulerAng, const char *);

#endif /* LG_LIGHT_H */
