/*
 *	LibGame - Copyright (C) Emmanuel Thomas-Maurin 2011-2025
 *	All rights reserved
 */

#ifndef LG_MISC_H
#define LG_MISC_H

#define CHECK_GL_E	lg_check_gl_errors(lg_log_time(), __FILE__ , __LINE__, __func__)

#define CHECK_GL_E_2	{if (lg_check_gl_errors(lg_log_time(), __FILE__, __LINE__, __func__) != LG_OK) exit(-1);}

#define STH_WRONG_HERE \
	{ \
		static int i = 0; \
		if (i == 0) { \
			INFO_OUT("VARO TÄSSÄ\n%s [%s: %d] %s(): jotain pitää korjata\n", UTF8_RW_ROUNDED_ARROW, __FILE__, __LINE__, __func__) \
			i++; \
		} \
	}

/* Not inclusive */
#define IS_IN(x, y, z)	(x > y && x < z ? TRUE : FALSE)

/* Inclusive */
#define IS_IN2(x, y, z)	(x >= y && x <= z ? TRUE : FALSE)

typedef enum {LG_LITTLE_ENDIAN, LG_BIG_ENDIAN} lg_endianness;

/*
 * A few conversions helpers
 *
 * 1 knot (KT) = 1 nautical mile (NM) / h
 */
#define FEET_TO_METERS		0.3048
#define METERS_TO_FEET		(1 / FEET_TO_METERS)
#define MILES_TO_KM		1.852
#define KM_TO_MILES		(1 / MILES_TO_KM)
#define KNOTS_TO_KMH		MILES_TO_KM
#define KMH_TO_KNOTS		KM_TO_MILES

/* Only for backward compatibility */
#define UTF8_RW_ROUNDED_ARROW	RW_ROUNDED_ARROW_UTF8

// === Tokens, UTF-8 hex codes and drawings ===

#define LEFT_ARROW_LIGHT_UTF8	"\u2190"	//	←
#define RIGHT_ARROW_LIGHT_UTF8	"\u2192"	//	→
#define UP_ARROW_LIGHT_UTF8	"\u2191"	//	↑
#define DOWN_ARROW_LIGHT_UTF8	"\u2193"	//	↓
#define RW_ROUNDED_ARROW_UTF8	"\u279c"	//	➜

/*
	BOX_N_UTF8

	1--2--3-----4
	5     |     |
	6-----7-----8
	|     |     |
	9----10----11
*/

#define BOX_1_UTF8	"\u250c"	//	┌
#define BOX_2_UTF8	"\u2500"        //	─
#define BOX_3_UTF8	"\u252c"        //	┬
#define BOX_4_UTF8	"\u2510"        //	┐
#define BOX_5_UTF8	"\u2502"        //	│
#define BOX_6_UTF8	"\u251c"        //	├
#define BOX_7_UTF8	"\u253c"        //	┼
#define BOX_8_UTF8	"\u2524"        //	┤
#define BOX_9_UTF8	"\u2514"        //	└
#define BOX_10_UTF8	"\u2534"        //	┴
#define BOX_11_UTF8	"\u2518"        //	┘

#define STACK_N_MAX_ELEMENTS	1024

/* Dynamic stacks */
typedef struct {
	int	*v;
	int	max_n;
	int	i;
} LG_Stack_i;

typedef struct {
	float	*v;
	int	max_n;
	int	i;
} LG_Stack_f;

void		lg_enable_transparency();

void		lg_disable_transparency();

int		lg_check_gl_errors(long long, const char *, int, const char *);

int		lg_check_gl_context();

lg_endianness	lg_get_arch_endianness();

float		lg_tv_diff(struct timeval *, struct timeval *, struct timeval *);

int 		delta_i(int);

float		delta_f(float);

zboolean	has_changed_i(int);

zboolean	has_changed_f(float);

void		lg_stack_show_full_empty_errors(zboolean);

LG_Stack_i	*lg_stack_i_new(int);

void		lg_stack_i_free(LG_Stack_i *);

zboolean	push_i(LG_Stack_i *, int);

zboolean	pop_i(LG_Stack_i *, int *);

LG_Stack_f	*lg_stack_f_new(int);

void		lg_stack_f_free(LG_Stack_f *);

zboolean	push_f(LG_Stack_f *, float);

zboolean	pop_f(LG_Stack_f *, float *);

float		lg_swap_float_values(float *, float *);

double		lg_swap_double_values(double *x, double *y);

int32_t		lg_swap_int32_values(int32_t *x, int32_t *y);

uint32_t	lg_swap_uint32_values(uint32_t *x, uint32_t *y);

int16_t		lg_swap_int16_values(int16_t *x, int16_t *y);

uint16_t	lg_swap_uint16_values(uint16_t *x, uint16_t *y);

int8_t		lg_swap_int8_values(int8_t *x, int8_t *y);

uint8_t		lg_swap_uint8_values(uint8_t *x, uint8_t *y);

void		debug_start_max_int_in_loop();

void		debug_max_int_in_loop(unsigned long, const char *, int);

void		debug_end_max_int_in_loop();

void		debug_start_max_float_in_loop();

void		debug_max_float_in_loop(float, const char *, int);

void		debug_end_max_float_in_loop();

int		lg_check_datatype_sizes();

void		lg_show_struct_sizes(zboolean);

int		lg_show_pixel_format_info_from_sdl_display_mode(SDL_DisplayMode *);

void		lg_show_utf8_drawings(zboolean);

#endif /* LG_MISC_H */
