/*
 *	LibGame - Copyright (C) Emmanuel Thomas-Maurin 2011-2025
 *	All rights reserved
 */

#ifndef LG_SHADER_PROGS_H
#define LG_SHADER_PROGS_H

#define LG_INVALID_SHADER_PROG_ID	500000

typedef struct {
	unsigned int	id;
} LG_ShaderProg;

/* Uniform locations */
typedef struct {
	int	world_m;
	int	view_m;
	int	proj_m;
	int	view_proj_m;
	int	model_view_m;		/* Or world_view_m */
	int	model_view_proj_m;	/* Or world_view_proj_m or MVP_matrix */
	int	texcoords;
	int	light_type;
	int	light_pos;
	int	light_dir;
	int	light_color;
	int	luminosity_k;
} LG_ShaderULocs;

/* Vertex Attrib Pointer params */
typedef struct {
	int32_t		index;
	const char	*attrib_name;
	uint32_t	num_components;			/* 1 to 4 */
	int32_t		type;				/* One of GL_BYTE, GL_UNSIGNED_BYTE, GL_SHORT, GL_UNSIGNED_SHORT, GL_FIXED, GL_FLOAT */
	zboolean	do_normalize;
	size_t		stride;				/* Byte offset between consecutive generic vertex attributes */
	void		*first_component_offset;	/* For instance: offsetof(Vertex_uv_n, u) */
} LG_VAP_params;

#define VAP_END		{-1, NULL, 0, 0, 0, 0, NULL}

LG_ShaderProg	LG_ShaderProg_create(const char **, const char **, LG_VAP_params *);

void		LG_ShaderProg_delete(LG_ShaderProg *);

void		LG_ShaderProg_use(LG_ShaderProg *, zboolean);

char		*LG_ShaderProg_load_src_from_file(const char *);

int		LG_ShaderProg_get_attrib_loc(LG_ShaderProg *, const char *);

int		LG_ShaderProg_get_uniform_loc(LG_ShaderProg *, const char *);

uint32_t	lg_load_shader(int, const char **);

void		lg_shader_info_log(uint32_t);

#endif /* LG_SHADER_PROGS_H */
