/*
 *	LibGame - Copyright (C) Emmanuel Thomas-Maurin 2011-2025
 *	All rights reserved
 */

#ifndef LG_TERRAIN_H
#define LG_TERRAIN_H

#define HEIGHTMAP_MAX_W		1024
#define HEIGHTMAP_MAX_H		1024

#define HEIGHTMAP_MIN_W		8
#define HEIGHTMAP_MIN_H		8

#define TERRAIN_BBOX_COLOR	"cyan"

typedef enum {
	RANDOM_NOISE,
	PERLIN_NOISE,
	OPENSIMPLEX2_NOISE,
	CELLULAR_NOISE		/* = VORONOI_NOISE */
	/*
	OPENSIMPLEX2S_NOISE
	VALUE_CUBIC_NOISE
	VALUE_NOISE
	*/
} lg_noise_type;

#define NORTH_BORDER	1
#define SOUTH_BORDER	2
#define WEST_BORDER	4
#define EAST_BORDER	8

/* Horizontally-centered and normalized */
typedef struct {
	Vertex_uv_n	*vbo_data;	/* VBO (with interleaved vertex data) - sizeof(Vertex_uv_n) = 24 */
	uint32_t	*ibo_data;	/* IBO (indices start at 0) - sizeof(uint32_t) = 4 */
	uint32_t	vbo_size;
	uint32_t	ibo_size;
	uint32_t	n_vertices;
	uint32_t	n_faces;
	uint32_t	n_indices;	/* Needed ? Always = n_faces * 3 */
	LG_Texture	*tex;
	uint16_t	elevation[HEIGHTMAP_MAX_W * HEIGHTMAP_MAX_H];	/* heightmap * 65535 * vert_scaling_k */
	uint16_t	grid_w;
	uint16_t	grid_h;
} LG_Terrain;

LG_Terrain		*lg_terrain_new(uint16_t, uint16_t, float, int, int, float, LG_Texture *);

LG_Terrain		*lg_terrain_new_from_heightmap(float *, uint16_t, uint16_t, float, LG_Texture *);

float			lg_terrain_get_elevation(LG_Terrain *, float, float, uint16_t, uint16_t);

void			lg_terrain_free(LG_Terrain *);

float			*lg_heightmap_generate(uint16_t, uint16_t, int, int, float);

void			lg_heightmaps_add(float *, float *, uint16_t, uint16_t, float);

void			lg_heightmap_flatten_border(float *, uint16_t, uint16_t, uint16_t, int);

void			lg_heightmap_apply_func(float *, uint16_t, uint16_t, float (*)(uint16_t, uint16_t, float));

LG_Mesh			*lg_terrain_to_mesh(LG_Terrain *);

Vertex_rgba		*lg_horiz_grid(uint16_t, int *, float, LG_Color_u);

// DEPRECATED
const Vertex		*lg_horiz_grid0();

const unsigned short	*lg_horiz_grid0_indices();

size_t			lg_sizeof_horiz_grid0();

size_t			lg_sizeof_horiz_grid0_indices();

#endif /* LG_TERRAIN_H */
