/*
 *	LibGame - Copyright (C) Emmanuel Thomas-Maurin 2011-2025
 *	All rights reserved
 */

#ifndef LG_WIN_H
#define LG_WIN_H

/* A few helpers for wins funcs: */
#define BLOCK		TRUE
#define NO_BLOCK	(!BLOCK)

#define WITH_FRAME	TRUE
#define FRAMELESS	(!WITH_FRAME)

#define SWAP_FB		TRUE
#define NO_SWAP		(!SWAP_FB)

#if defined(LINUX_V) || defined(WIN32_V)
  #define PRESS_OR_TAP_ANYTHING		"Press any key ..."
#elif defined(ANDROID_V)
  #define PRESS_OR_TAP_ANYTHING		"Tap anywhere to continue ..."
#endif

/* Fully transparent colors actually */
#define UNUSED_SDL_COLOR		((SDL_Color){0, 0, 0, 0})
#define UNUSED_LG_COLOR_U		((LG_Color_u){0, 0, 0, 0})

#define TMP_STR_MAXLEN			1023

#define H_PADDING			12
#define V_PADDING			8
#define BORDER_WIDTH			2

#define LG_WIN_TXT_NBSP		'~'

/* TODO: what if win text is over 24 lines ? */
#define LG_WIN_N_LINES_MAX		24

#define LG_LEFT_MARGIN			5
#define LG_BOTTOM_MARGIN		5

#define LG_WIN_TIMESTAMP_MAXLEN		(64 - 1)

typedef struct {
	int		id;		/* (So far) 0 -> OK, -1 -> invalid win */
	LG_Texture	*tex;		/* Used inside new/open/close/free_win(), don't modify elsewhere */
	Rec2Di		text_rect;
	int		x;
	int		y;
	/* === Don't modify === */
	int		w;
	int		h;
	int		last_open_x;
	int		last_open_y;
	char		timestamp[LG_WIN_TIMESTAMP_MAXLEN + 1];
} LG_Window;

/* Wins colors */
#define ALL_WINS_BG_COLOR		"dark-grey2"
#define ALL_WINS_BORDER_COLOR		"dark-grey2"

#define INFO_WIN_TEXT_COLOR		"white"
#define INFO_WIN_BG_COLOR		ALL_WINS_BG_COLOR
#define INFO_WIN_BORDER_COLOR		ALL_WINS_BORDER_COLOR
#define INFO_WIN_FONT
#define INFO_WIN_STYLE

#define QUESTION_WIN_TEXT_COLOR		"white"
#define QUESTION_WIN_BG_COLOR		ALL_WINS_BG_COLOR
#define QUESTION_WIN_BORDER_COLOR	ALL_WINS_BORDER_COLOR
#define QUESTION_WIN_FONT
#define QUESTION_WIN_STYLE

#define WARNING_WIN_TEXT_COLOR		"orange"
#define WARNING_WIN_BG_COLOR		ALL_WINS_BG_COLOR
#define WARNING_WIN_BORDER_COLOR	ALL_WINS_BORDER_COLOR
#define WARNING_WIN_FONT
#define WARNING_WIN_STYLE

#define ERROR_WIN_TEXT_COLOR		"red"
#define ERROR_WIN_BG_COLOR		ALL_WINS_BG_COLOR
#define ERROR_WIN_BORDER_COLOR		ALL_WINS_BORDER_COLOR
#define ERROR_WIN_FONT
#define ERROR_WIN_STYLE

/*
 * Default, modifiable colors
 * Never used:
 * 	win_colors.warning[BG_I]
 * 	win_colors.warning[BORDER_I]
 * 	win_colors.error[BG_I],
 * 	win_colors.error[BORDER_I]
 * Almost never used:
 * 	win_colors.question[BG_I]
 * 	win_colors.question[BORDER_I]
 */

typedef struct {
	LG_Color_u	info[3];	/* Text, bg, and border */
	LG_Color_u	question[3];
	LG_Color_u	warning[3];
	LG_Color_u	error[3];
} LG_WinColors;

/* and for each their indices */
enum {TEXT_I, BG_I, BORDER_I};

/* Fonts things */
#define N_FONTS_MAX	256

typedef struct {
	LG_String	*name;
	LG_String	*relative_path;		/* Path inside the fonts folder, not relative if symlink */
	int		size;
	TTF_Font	*f;
} LG_Font;

/* Font description */
typedef struct {
	const char	*name;
	const char	*relative_path;		/* Path inside the fonts folder, not relative if symlink */
	int		size;
} LG_Font_Des;

typedef struct {
	LG_Color_u	text_color;
	zboolean	with_frame;
	LG_Color_u	bg_color;
	LG_Color_u	border_color;
	const char	*font;
} LG_Win_Style;

void		lg_info_win_no_block(const char *, zboolean, zboolean);

void		lg_info_win(const char *, zboolean, zboolean);

int		lg_question_win(const char *);				/* swap_fb */

int		lg_question_win_s(const char *, LG_Win_Style *);	/* swap_fb */

void		lg_warning_win(const char *);				/* swap_fb */

void		lg_error_win(const char *);				/* swap_fb */

char		*lg_entry_win(const char *, int, int);			/* swap_fb */

int		lg_win_open_centered(LG_Window *);

LG_Window	lg_win(const char *, LG_Color_u, zboolean, LG_Color_u, LG_Color_u, TTF_Font *);

int		lg_win_open(LG_Window *, int, int);

int		lg_win_close(LG_Window *);

void		lg_win_free_tex(LG_Window *);

zboolean	is_lg_win(LG_Window *);

int		lg_get_win_w(LG_Window *);

int		lg_get_win_h(LG_Window *);

LG_WinColors	*lg_get_win_colors();

void		lg_set_default_win_colors();

char		*lg_wrap_lines(const char *, TTF_Font *);

void		lg_fonts_list_init();

zboolean	lg_fonts_list_is_initialized();

zboolean	lg_font_add_to_list(const char *, const char *, int);

LG_Font		*lg_font_get(const char *);

TTF_Font	*lg_font_get_ttf(const char *);

void		lg_font_free(const char *);

void		lg_font_free_all();

void		lg_font_list_all();

void		lg_font_info(LG_Font *);

zboolean	lg_font_load_array(LG_Font_Des [], int);

void		lg_font_list_array(LG_Font_Des [], int);

#endif /* LG_WIN_H */
